// Classes pour imp�dance de circuits R,L et C (sans boucle)
// Insa Lyon Dept GE.
// 2013

#include <iostream>
#include <vector>
#include <string>
#include <math.h>

using namespace std;

/* CLASSE COMPLEXE */

class Complexe
{
public:
	double Reel;
	double Imag;

	void Accumulate(Complexe const& z);
	Complexe Inverse(void) const;
	void Print(void) const;
};

void Complexe::Accumulate(Complexe const& z)
{
	Reel += z.Reel;
	Imag += z.Imag;
}

Complexe Complexe::Inverse(void) const
{
	Complexe r;
	double d;

	d = Reel * Reel + Imag * Imag;
	r.Reel = Reel / d;
	r.Imag = -Imag / d;
	return (r);
}

void Complexe::Print(void) const
{
	if (Imag < 0.0)
		cout << " = " << Reel << " - " << Imag << "i" << endl;
	else
		cout << " = " << Reel << " + " << Imag << "i" << endl;
}


/* CLASSE COMPONENT */
class Component
{
	public:
		double	Valeur;
		string	Name;

		Component(string const& name, double const& v);
		void Reset(void);
		Component* GetPrevious(void) const;
		Component* GetNext(void) const;
		void SetPrevious(Component *comp);
		virtual Complexe Evaluate(double const& freq) const = 0;
		virtual void Print(void) const;

	private:
		Component	*Previous;
		Component	*Next;
};

Component::Component(string const& name, double const& v) : Name(name), Valeur(v)
{
	Reset();
}

void Component::Reset(void)
{
	Previous = NULL;
	Next = NULL;
}

Component*	Component::GetPrevious(void) const
{
	return (Previous);
}

Component*	Component::GetNext(void) const
{
	return (Next);
}

void Component::SetPrevious(Component *comp)
{
	//unlink
	if (Previous != NULL)
		Previous->Next = comp;
	// make new link
	Previous = comp;
	Previous->Next = this;
}

void Component::Print(void) const
{
	cout << Name << "(" << Valeur << ")";
}


/* CLASSE RESISTOR */
class Resistor : public Component
{
	public:
		Resistor(string const& name, double const& v);
		Complexe Evaluate(double const& freq) const;
};

Resistor::Resistor(string const& name, double const& v) : Component(name, v)
{

}

Complexe Resistor::Evaluate(double const& freq) const
{
 //TODO
 
}



/* CLASSE CAPACITOR */
class Capacitor : public Component
{
	public:
		Capacitor(string const& name, double const& v);
		Complexe Evaluate(double const& freq) const;
};

Capacitor::Capacitor(string const& name, double const& v) : Component(name, v)
{

}

Complexe Capacitor::Evaluate(double const& freq) const
{
//TODO

}

/* CLASSE INDUCTOR */

class Inductor : public Component
{
	public:
		Inductor(string const& name, double const& v);
		Complexe Evaluate(double freq) const;
};

Inductor::Inductor(string const& name, double const& v) : Component(name, v)
{

}

Complexe Inductor::Evaluate(double freq) const
{
// TODO

}

/* CLASSE IMPEDANCE */

class Impedance : public Component
{
//TODO

public:
	//TODO

	
	
		
	void Print() const;
};


void Impedance::Print() const
		{
		cout<<Name<<"[ ";
		if( !Branch.empty() )
		for(unsigned int i=0; i<Branch.size(); i++)
			{		
			Component *comp = Branch[i];
			do
				{
				comp->Print();
				comp = comp->GetNext();
				if( comp != NULL ) cout<<" + ";
				}while( comp != NULL );
			if( (Branch.size() > 1) && (i < Branch.size()-1))
				 cout<< " // ";
			}
		cout<<"]";
		}


/* MAIN */

int main(void)
{
// Premier circuit
Resistor R1("R1",50), R2("R2",100) , R3("R3",100);
Impedance Z1("",0);
Impedance Z2("",0);
Z2.AddBranch(&R2);
Z2.AddBranch(&R3);
Z2.SetPrevious(&R1);
Z1.AddBranch(&R1);

Z1.Print();
Z1.Evaluate(0).Print();
cin.get();


 // second circuit
/*
R1.Reset(); R2.Reset(); R3.Reset();
Capacitor	C1("C1", 0.001);
Inductor	L1("L1", 0.002);

Impedance RLC("RLC",0);
RLC.AddBranch(&R1);
RLC.AddBranch(&C1);
RLC.AddBranch(&L1);
RLC.Print();
cout<< endl;
for( double f=0.01;f<100;f+=10)
	{
	cout<< "Z("<< f << ") ";
	RLC.Evaluate(f).Print();
	}
cin.get();
*/
return 0;
}